/*
 * Copyright (c) Microsoft Corporation.
 * Licensed under the MIT License
 */

#pragma once

# include "mscrypt.h"
# include <openssl/bio.h>

#ifdef  __cplusplus
extern "C" {
#endif


#define MSCRYPTP_VERSION_1          1

#define MSCRYPTP_TRACE_LOG_TEST_EXECUTE_FLAG    0x2
#define MSCRYPTP_TRACE_LOG_VERBOSE_EXECUTE_FLAG 0x4
void MSCRYPTP_set_execute_flags(
    int flags);

// Call the following to redirect the trace log output
// from stdout to the specified file.
void MSCRYPTP_set_trace_log_filename(
    const char *filename);



typedef struct MSCRYPT_GDBUS_shared_mem_st MSCRYPT_GDBUS_SHARED_MEM;
struct MSCRYPT_shared_mem_st {
    uuid_t                      correlationId;
    int                         memLength;
    unsigned char               *memBytes;
    MSCRYPT_GDBUS_SHARED_MEM    *gdbus;
};

typedef struct MSCRYPT_pfx_ctx_st MSCRYPT_PFX_CTX;
typedef struct MSCRYPT_GBUS_pfx_ctx_st MSCRYPT_GDBUS_PFX_CTX;
struct MSCRYPT_pfx_ctx_st {
    uuid_t                      correlationId;
    void                        *pkey;
    MSCRYPT_GDBUS_PFX_CTX       *gdbus;
};


#ifndef MSCRYPT_ROOT_DIR
#ifdef MSCRYPT_TEST_WINDOWS
#define MSCRYPT_ROOT_DIR ""
#else
#define MSCRYPT_ROOT_DIR "/var/opt/msft/kmpp"
#endif
#endif

#ifndef MSCRYPT_PRIVATE_ROOT_DIR
#ifdef MSCRYPT_TEST_WINDOWS
#define MSCRYPT_PRIVATE_ROOT_DIR ""
#else
#define MSCRYPT_PRIVATE_ROOT_DIR "/var/opt/msft/kmpp-private"
#endif
#endif

#define MSCRYPT_CERTS_DIR MSCRYPT_ROOT_DIR "/certs"

#ifndef MSCRYPT_INSTALL_IMAGE_DIR
#ifdef MSCRYPT_TEST_WINDOWS
#define MSCRYPT_INSTALL_IMAGE_DIR ""
#else
#define MSCRYPT_INSTALL_IMAGE_DIR "/opt/msft/kmpp"
#endif
#endif


void MSCRYPTP_set_default_dir(
    const char *defaultCertArea,
    const char *defaultCertDir);

const char *MSCRYPTP_get_default_cert_area();
const char *MSCRYPTP_get_default_private_area();
const char *MSCRYPTP_get_default_cert_dir();
const char *MSCRYPTP_get_install_image_dir();

// Includes NULL terminator character
#define MSCRYPTP_BASE64_ENCODE_LENGTH(inLength) ((((inLength + 3 - 1) / 3) * 4) + 1)

// Returns BIO_s_mem().
// Ensures a NULL terminator is always appended to the read file contents.
BIO *MSCRYPTP_read_file_string(
    const uuid_t correlationId,
    const char *fileName,
    int disableTraceLog,
    char **str);

// Returns number of decode bytes. For a decode error returhs -1.
int MSCRYPTP_base64_decode(
    const uuid_t correlationId,
    const char *str,
    unsigned char **bytes);     // MSCRYPT_free()

// Converts binary bytes to NULL terminated ascii hex characters.
// Returned hex needs (len * 2 + 1) characters
void MSCRYPTP_bytes_to_hex(
    int len,
    const unsigned char *pb,
    char *hex);

void MSCRYPTP_get_verify_cert_ctx_correlationId(
    MSCRYPT_VERIFY_CERT_CTX *ctx,
    uuid_t correlationId);

//
// Wrapper for openSSL!X509_verify_cert().
//
// Return:
//  +1 - Success with complete chain of certificates to a trusted root
//  -1 - Success with chain error. Might be missing intermediate certs.
//       *verifyChainError is updated with X509_V_ERR_* error defined
//       in x509_vfy.h.
//   0 - Error, unable to build chain
int MSCRYPTP_X509_verify_cert(
    MSCRYPT_VERIFY_CERT_CTX *ctx,
    X509_STORE_CTX *storeCtx,
    int mscryptFlags,
    int *verifyChainError);

//
// MSCRYPTP_trace_log_* functions
//  Implementation: mscryptsupport.c
// 

extern int MSCRYPTP_traceLogTest;
extern int MSCRYPTP_traceLogVerbose;

#define MSCRYPTP_TRACELOG_PARA_LENGTH       256
#define MSCRYPTP_TRACELOG_ERROR_LENGTH      256

#define MSCRYPTP_TRACELOG_VERBOSE_FLAG      0x1
#define MSCRYPTP_TRACELOG_WARNING_FLAG      0x2

#define MSCRYPTP_ENGINE_TITLE               "MsCryptEngine"
#define MSCRYPTP_SUPPORT_TITLE              "MsCryptSupport"
#define MSCRYPTP_MEMORY_ALLOC_TITLE         "MsCryptMemoryAlloc"
#define MSCRYPTP_IMPORT_PFX_TITLE           "MsCryptImportPfx"
#define MSCRYPTP_OPEN_PFX_TITLE             "MsCryptOpenPfx"
#define MSCRYPTP_CLOSE_PFX_TITLE            "MsCryptClosePfx"
#define MSCRYPTP_HELPER_PFX_TITLE           "MsCryptHelperPfx"
#define MSCRYPTP_PFX_SECRET_TITLE           "MsCryptPfxSecret"
#define MSCRYPTP_IMPORT_TRUSTED_TITLE       "MsCryptImportTrusted"
#define MSCRYPTP_REMOVE_TRUSTED_TITLE       "MsCryptRemoveTrusted"
#define MSCRYPTP_ENUM_TRUSTED_TITLE         "MsCryptEnumTrusted"
#define MSCRYPTP_IS_TRUSTED_TITLE           "MsCryptIsTrusted"
#define MSCRYPTP_IMPORT_DISALLOWED_TITLE    "MsCryptImportDisallowed"
#define MSCRYPTP_REMOVE_DISALLOWED_TITLE    "MsCryptRemoveDisallowed"
#define MSCRYPTP_ENUM_DISALLOWED_TITLE      "MsCryptEnumDisallowed"
#define MSCRYPTP_IS_DISALLOWED_TITLE        "MsCryptIsDisallowed"
#define MSCRYPTP_VERIFY_CERT_TITLE          "MsCryptVerifyCert"
#define MSCRYPTP_HELPER_CERT_TITLE          "MsCryptHelperCert"
#define MSCRYPTP_CREATE_SELF_SIGN_TITLE     "MsCryptCreateSelfSign"
#define MSCRYPTP_RSA_ENCRYPT_TITLE          "MsCryptRsaEncrypt"
#define MSCRYPTP_RSA_DECRYPT_TITLE          "MsCryptRsaDecrypt"
#define MSCRYPTP_ECC_SIGN_TITLE             "MsCryptEccSign"
#define MSCRYPTP_RSA_SIGN_TITLE             "MsCryptRsaSign"
#define MSCRYPTP_PKEY_RSA_SIGN_TITLE        "MsCryptPkeyRsaSign"
#define MSCRYPTP_CURL_TITLE                 "MsCryptCurl"
#define MSCRYPTP_TEST_TITLE                 "MsCryptTest"
#define MSCRYPTP_SERVICE_TITLE              "MsCryptService"
#define MSCRYPTP_GDBUS_CLIENT_TITLE         "MsCryptGdbusClient"
#define MSCRYPTP_ERROR_STACK_TITLE          "MsCryptErrorStack"

void _MSCRYPTP_trace_log_output(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    const char *error,
    const char *paraFormat,
    va_list paraArgs);

void _MSCRYPTP_trace_log_para(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    const char *format, ...);
#define MSCRYPTP_trace_log_para(correlationId, flags, title, loc, ...) \
    _MSCRYPTP_trace_log_para(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc, __VA_ARGS__)

void _MSCRYPTP_trace_log(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc);
#define MSCRYPTP_trace_log(correlationId, flags, title, loc) \
    _MSCRYPTP_trace_log(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc)

void _MSCRYPTP_trace_log_error_para(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    const char *errStr,
    const char *format, ...);
#define MSCRYPTP_trace_log_error_para(correlationId, flags, title, loc, errStr,  ...) \
    _MSCRYPTP_trace_log_error_para(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc, errStr, __VA_ARGS__)

void _MSCRYPTP_trace_log_error(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    const char *errStr);
#define MSCRYPTP_trace_log_error(correlationId, flags, title, loc, errStr) \
    _MSCRYPTP_trace_log_error(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc, errStr)

void _MSCRYPTP_trace_log_openssl_error_para(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    const char *format, ...);
#define MSCRYPTP_trace_log_openssl_error_para(correlationId, flags, title, loc, ...) \
    _MSCRYPTP_trace_log_openssl_error_para(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc, __VA_ARGS__)

void _MSCRYPTP_trace_log_openssl_error(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc);
#define MSCRYPTP_trace_log_openssl_error(correlationId, flags, title, loc) \
    _MSCRYPTP_trace_log_openssl_error(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc)


void _MSCRYPTP_trace_log_openssl_verify_cert_error_para(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    int err,
    const char *format, ...);
#define MSCRYPTP_trace_log_openssl_verify_cert_error_para(correlationId, flags, title, loc, err, ...) \
    _MSCRYPTP_trace_log_openssl_verify_cert_error_para(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc, err, __VA_ARGS__)

void _MSCRYPTP_trace_log_openssl_verify_cert_error(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    int err);
#define MSCRYPTP_trace_log_openssl_verify_cert_error(correlationId, flags, title, loc, err) \
    _MSCRYPTP_trace_log_openssl_verify_cert_error(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc, err)

void _MSCRYPTP_trace_log_errno_para(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    int err,
    const char *format, ...);
#define MSCRYPTP_trace_log_errno_para(correlationId, flags, title, loc, err, ...) \
    _MSCRYPTP_trace_log_errno_para(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc, err, __VA_ARGS__)

void _MSCRYPTP_trace_log_errno(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    int err);
#define MSCRYPTP_trace_log_errno(correlationId, flags, title, loc, err) \
    _MSCRYPTP_trace_log_errno(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc, err)


////~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~////
////                                Deprecated section - Removed APIs  !!!!!!!                       ////
////~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~////

// Please note that the following APIs will soon be deprecated.
// It is recommended that you update your applications and remove those functions as soon as possible.

#define MSCRYPTP_TEST_PFX_SECRET_EXECUTE_FLAG (0x16) 

typedef struct MSCRYPT_rsa_sign_st MSCRYPT_RSA_SIGN;
struct MSCRYPT_rsa_sign_st {
    int32_t                     type;       // message digest algorithm
    uint32_t                    m_len;      // message digest length
};

typedef struct MSCRYPT_evp_pkey_sign_st MSCRYPT_EVP_PKEY_SIGN;
struct MSCRYPT_evp_pkey_sign_st {
    uint64_t                    tbslen;     // length of the data to be signed
    uint64_t                    siglen;     // length of the signature
    int32_t                     saltlen;    // RSA PSS salt length
    int32_t                     sigmdtype;  // nid of signature algorithm
    int32_t                     mgfmdtype;  // nid of MGF1 digest
    int32_t                     getMaxLen;  // indicate to only calculate the max siglen 
};

__attribute__ ((deprecated))
int MSCRYPTP_load_pfx_certs(
    const uuid_t correlationId,
    int pfxLength,
    const unsigned char *pfxBytes,
    X509 **cert,
    STACK_OF(X509) **ca);       // Optional

__attribute__ ((deprecated))
int MSCRYPTP_pem_from_certs(
    const uuid_t correlationId,
    X509 *cert,                     // Optional
    STACK_OF(X509) *ca,             // Optional
    int *pemCertLength,             // Excludes NULL terminator
    char **pemCert);                // MSCRYPT_free()                   
    
__attribute__ ((deprecated))
const char *MSCRYPTP_get_cert_ctrl_title(
    int ctrl,
    int location);

__attribute__ ((deprecated))
int MSCRYPTP_install_image_certs(
    const uuid_t correlationId);
    
__attribute__ ((deprecated))
int MSCRYPTP_create_pfx_secret(
    const uuid_t correlationId);

////~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~////


#ifdef  __cplusplus
}
#endif
